@@ -16,6 +16,7 @@ def eaSimple(
     callbacks=None,
     verbose=True,
     estimator=None,
+    **kwargs,
 ):
     """
     The base implementation is directly taken from: https://github.com/DEAP/deap/blob/master/deap/algorithms.py
@@ -205,6 +206,7 @@ def eaMuPlusLambda(
     callbacks=None,
     verbose=True,
     estimator=None,
+    **kwargs,
 ):
     """
     The base implementation is directly taken from: https://github.com/DEAP/deap/blob/master/deap/algorithms.py
@@ -316,7 +318,6 @@ def eaMuPlusLambda(
 
     for gen in range(1, ngen + 1):
         try:
-
             # Vary the population
             offspring = varOr(population, toolbox, lambda_, cxpb.step(), mutpb.step())
 
@@ -390,6 +391,7 @@ def eaMuCommaLambda(
     callbacks=None,
     verbose=True,
     estimator=None,
+    **kwargs,
 ):
     """
     The base implementation is directly taken from: https://github.com/DEAP/deap/blob/master/deap/algorithms.py
@@ -504,7 +506,6 @@ def eaMuCommaLambda(
 
     for gen in range(1, ngen + 1):
         try:
-
             # Vary the population
             offspring = varOr(population, toolbox, lambda_, cxpb.step(), mutpb.step())
 
@@ -564,3 +565,10 @@ def eaMuCommaLambda(
     eval_callbacks(**callbacks_end_args)
 
     return population, logbook, n_gen
+
+
+algorithms_factory = {
+    "eaSimple": eaSimple,
+    "eaMuPlusLambda": eaMuPlusLambda,
+    "eaMuCommaLambda": eaMuCommaLambda,
+}
