@@ -6,9 +6,12 @@ import numpy as np
 from deap import base, creator, tools
 from sklearn.base import clone
 from sklearn.model_selection import cross_validate
-from sklearn.base import is_classifier, is_regressor
+from sklearn.base import is_classifier, is_regressor, BaseEstimator, MetaEstimatorMixin
+from sklearn.feature_selection import SelectorMixin
 from sklearn.utils import check_X_y
 from sklearn.utils.validation import check_is_fitted
+from sklearn.utils.metaestimators import available_if
+from sklearn.feature_selection._from_model import _estimator_has
 from sklearn.metrics import check_scoring
 from sklearn.exceptions import NotFittedError
 from sklearn.model_selection._search import BaseSearchCV
@@ -17,14 +20,15 @@ from sklearn.metrics._scorer import _check_multimetric_scoring
 
 from .parameters import Algorithms, Criteria
 from .space import Space
-from .algorithms import eaSimple, eaMuPlusLambda, eaMuCommaLambda
+from .algorithms import algorithms_factory
 from .callbacks.validations import check_callback
 from .schedules.validations import check_adapter
 from .utils.cv_scores import (
     create_gasearch_cv_results_,
     create_feature_selection_cv_results_,
 )
-from .utils.random import weighted_choice
+from .utils.random import weighted_bool_individual
+from .utils.tools import cxUniform, mutFlipBit
 
 
 class GASearchCV(BaseSearchCV):
@@ -232,8 +236,8 @@ class GASearchCV(BaseSearchCV):
         return_train_score=False,
         log_config=None,
     ):
-
         self.estimator = clone(estimator)
+        self.estimator_ = None
         self.toolbox = base.Toolbox()
         self.cv = cv
         self.scoring = scoring
@@ -282,14 +286,10 @@ class GASearchCV(BaseSearchCV):
 
         # Check that the estimator is compatible with scikit-learn
         if not is_classifier(self.estimator) and not is_regressor(self.estimator):
-            raise ValueError(
-                f"{self.estimator} is not a valid Sklearn classifier or regressor"
-            )
+            raise ValueError(f"{self.estimator} is not a valid Sklearn classifier or regressor")
 
         if criteria not in Criteria.list():
-            raise ValueError(
-                f"Criteria must be one of {Criteria.list()}, got {criteria} instead"
-            )
+            raise ValueError(f"Criteria must be one of {Criteria.list()}, got {criteria} instead")
         # Minimization is handle like an optimization problem with a change in the score sign
         elif criteria == Criteria.max.value:
             self.criteria_sign = 1.0
@@ -299,10 +299,10 @@ class GASearchCV(BaseSearchCV):
         # Saves the param_grid and computes some extra properties in the same object
         self.space = Space(param_grid)
 
-        if len(self.space) == 1:
+        if len(self.space) == 1:  # pragma: no cover
             warnings.warn(
                 "Warning, only one parameter was provided to the param_grid, the optimization routine "
-                "might not have effect, it's advised to use at least 2 parameters"
+                "might not have effect or it could lead to errors, it's advised to use at least 2 parameters"
             )
 
         super(GASearchCV, self).__init__(
@@ -342,9 +342,7 @@ class GASearchCV(BaseSearchCV):
             n=IND_SIZE,
         )
 
-        self.toolbox.register(
-            "population", tools.initRepeat, list, self.toolbox.individual
-        )
+        self.toolbox.register("population", tools.initRepeat, list, self.toolbox.individual)
 
         if len(self.space) == 1:
             sampler = list(self.space.param_grid.values())[0]
@@ -358,9 +356,7 @@ class GASearchCV(BaseSearchCV):
 
         self.toolbox.register("mutate", self.mutate)
         if self.elitism:
-            self.toolbox.register(
-                "select", tools.selTournament, tournsize=self.tournament_size
-            )
+            self.toolbox.register("select", tools.selTournament, tournsize=self.tournament_size)
         else:
             self.toolbox.register("select", tools.selRoulette)
 
@@ -379,7 +375,7 @@ class GASearchCV(BaseSearchCV):
 
     def mutate(self, individual):
         """
-        This function is responsible of changed a randomly selected parameter from an individual
+        This function is responsible for change a randomly selected parameter from an individual
         Parameters
         ----------
         individual: Individual object
@@ -408,7 +404,6 @@ class GASearchCV(BaseSearchCV):
         ----------
         individual: Individual object
             The individual (set of hyperparameters) that is being evaluated
-
         Returns
         -------
             The fitness value of the estimator candidate, corresponding to the cv-score
@@ -457,9 +452,7 @@ class GASearchCV(BaseSearchCV):
             current_generation_params[f"test_{metric}"] = cv_results[f"test_{metric}"]
 
             if self.return_train_score:
-                current_generation_params[f"train_{metric}"] = cv_results[
-                    f"train_{metric}"
-                ]
+                current_generation_params[f"train_{metric}"] = cv_results[f"train_{metric}"]
 
         index = len(self.logbook.chapters["parameters"])
         current_generation_params = {"index": index, **current_generation_params}
@@ -487,7 +480,6 @@ class GASearchCV(BaseSearchCV):
             One or a list of the callbacks methods available in
             :class:`~sklearn_genetic.callbacks`.
             The callback is evaluated after fitting the estimators from the generation 1.
-
         """
 
         self.X_ = X
@@ -517,9 +509,7 @@ class GASearchCV(BaseSearchCV):
         self._register()
 
         # Optimization routine from the selected evolutionary algorithm
-        pop, log, n_gen = self._select_algorithm(
-            pop=self._pop, stats=self._stats, hof=self._hof
-        )
+        pop, log, n_gen = self._select_algorithm(pop=self._pop, stats=self._stats, hof=self._hof)
 
         # Update the _n_iterations value as the algorithm could stop earlier due a callback
         self._n_iterations = n_gen
@@ -541,22 +531,22 @@ class GASearchCV(BaseSearchCV):
 
         # Imitate the logic of scikit-learn refit parameter
         if self.refit:
-            self.best_index_ = self.cv_results_[
-                f"rank_test_{self.refit_metric}"
-            ].argmin()
-            self.best_score_ = self.cv_results_[f"mean_test_{self.refit_metric}"][
-                self.best_index_
-            ]
+            self.best_index_ = self.cv_results_[f"rank_test_{self.refit_metric}"].argmin()
+            self.best_score_ = self.cv_results_[f"mean_test_{self.refit_metric}"][self.best_index_]
             self.best_params_ = self.cv_results_["params"][self.best_index_]
 
             self.estimator.set_params(**self.best_params_)
 
             refit_start_time = time.time()
-            self.estimator.fit(self.X_, self.y_)
+            self.estimator.fit(
+                self.X_,
+                self.y_,
+            )
             refit_end_time = time.time()
             self.refit_time_ = refit_end_time - refit_start_time
 
             self.best_estimator_ = self.estimator
+            self.estimator_ = self.best_estimator_
 
             # hof keeps the best params according to the fitness value
             # To be consistent with self.best_estimator_, if more than 1 model gets the
@@ -597,40 +587,9 @@ class GASearchCV(BaseSearchCV):
             The number of generations that the evolutionary algorithm ran
         """
 
-        if self.algorithm == Algorithms.eaSimple.value:
-
-            pop, log, gen = eaSimple(
-                pop,
-                self.toolbox,
-                cxpb=self.crossover_adapter,
-                stats=stats,
-                mutpb=self.mutation_adapter,
-                ngen=self.generations,
-                halloffame=hof,
-                callbacks=self.callbacks,
-                verbose=self.verbose,
-                estimator=self,
-            )
-
-        elif self.algorithm == Algorithms.eaMuPlusLambda.value:
-
-            pop, log, gen = eaMuPlusLambda(
-                pop,
-                self.toolbox,
-                mu=self.population_size,
-                lambda_=2 * self.population_size,
-                cxpb=self.crossover_adapter,
-                stats=stats,
-                mutpb=self.mutation_adapter,
-                ngen=self.generations,
-                halloffame=hof,
-                callbacks=self.callbacks,
-                verbose=self.verbose,
-                estimator=self,
-            )
-
-        elif self.algorithm == Algorithms.eaMuCommaLambda.value:
-            pop, log, gen = eaMuCommaLambda(
+        selected_algorithm = algorithms_factory.get(self.algorithm, None)
+        if selected_algorithm:
+            pop, log, gen = selected_algorithm(
                 pop,
                 self.toolbox,
                 mu=self.population_size,
@@ -720,7 +679,7 @@ class GASearchCV(BaseSearchCV):
         return self._n_iterations
 
 
-class GAFeatureSelectionCV(BaseSearchCV):
+class GAFeatureSelectionCV(MetaEstimatorMixin, SelectorMixin, BaseEstimator):
     """
     Evolutionary optimization for feature selection.
 
@@ -881,11 +840,15 @@ class GAFeatureSelectionCV(BaseSearchCV):
     best_features_ : list
         List of bool, each index represents one feature in the same order the data was fed.
         1 means the feature was selected, 0 means the features was discarded.
+    support_ : list
+        The mask of selected features.
     scorer_ : function or a dict
         Scorer function used on the held out data to choose the best
         parameters for the model.
     n_splits_ : int
         The number of cross-validation splits (folds/iterations).
+    n_features_in_ : int
+        Number of features seen (selected) during fit.
     refit_time_ : float
         Seconds used for refitting the best model on the whole dataset.
         This is present only if ``refit`` is not False.
@@ -914,8 +877,8 @@ class GAFeatureSelectionCV(BaseSearchCV):
         return_train_score=False,
         log_config=None,
     ):
-
         self.estimator = clone(estimator)
+        self.estimator_ = None
         self.toolbox = base.Toolbox()
         self.cv = cv
         self.scoring = scoring
@@ -947,6 +910,7 @@ class GAFeatureSelectionCV(BaseSearchCV):
         self.features_proportion = None
         self.callbacks = None
         self.best_features_ = None
+        self.support_ = None
         self.best_estimator_ = None
         self._pop = None
         self._stats = None
@@ -964,31 +928,16 @@ class GAFeatureSelectionCV(BaseSearchCV):
 
         # Check that the estimator is compatible with scikit-learn
         if not is_classifier(self.estimator) and not is_regressor(self.estimator):
-            raise ValueError(
-                f"{self.estimator} is not a valid Sklearn classifier or regressor"
-            )
+            raise ValueError(f"{self.estimator} is not a valid Sklearn classifier or regressor")
 
         if criteria not in Criteria.list():
-            raise ValueError(
-                f"Criteria must be one of {Criteria.list()}, got {criteria} instead"
-            )
+            raise ValueError(f"Criteria must be one of {Criteria.list()}, got {criteria} instead")
         # Minimization is handle like an optimization problem with a change in the score sign
         elif criteria == Criteria.max.value:
             self.criteria_sign = 1.0
         elif criteria == Criteria.min.value:
             self.criteria_sign = -1.0
 
-        super(GAFeatureSelectionCV, self).__init__(
-            estimator=estimator,
-            scoring=scoring,
-            n_jobs=n_jobs,
-            refit=refit,
-            cv=cv,
-            verbose=verbose,
-            pre_dispatch=pre_dispatch,
-            error_score=error_score,
-        )
-
     def _register(self):
         """
         This function is the responsible for registering the DEAPs necessary methods
@@ -997,38 +946,27 @@ class GAFeatureSelectionCV(BaseSearchCV):
 
         # Criteria sign to set max or min problem
         # And -1.0 as second weight to minimize number of features
-        self.creator.create(
-            "FitnessMax", base.Fitness, weights=[self.criteria_sign, -1.0]
-        )
+        self.creator.create("FitnessMax", base.Fitness, weights=[self.criteria_sign, -1.0])
         self.creator.create("Individual", list, fitness=creator.FitnessMax)
 
         # Register the array to choose the features
         # Each binary value represents if the feature is selected or not
 
-        if self.features_proportion:
-            self.toolbox.register("features", weighted_choice, self.features_proportion)
-        else:
-            self.toolbox.register("features", random.randint, 0, 1)
-
         self.toolbox.register(
             "individual",
-            tools.initRepeat,
+            weighted_bool_individual,
             creator.Individual,
-            self.toolbox.features,
-            n=self.n_features,
+            weight=self.features_proportion,
+            size=self.n_features,
         )
 
-        self.toolbox.register(
-            "population", tools.initRepeat, list, self.toolbox.individual
-        )
+        self.toolbox.register("population", tools.initRepeat, list, self.toolbox.individual)
 
-        self.toolbox.register("mate", tools.cxUniform, indpb=0.05)
-        self.toolbox.register("mutate", tools.mutFlipBit, indpb=0.05)
+        self.toolbox.register("mate", cxUniform, indpb=self.crossover_adapter.current_value)
+        self.toolbox.register("mutate", mutFlipBit, indpb=self.mutation_adapter.current_value)
 
         if self.elitism:
-            self.toolbox.register(
-                "select", tools.selTournament, tournsize=self.tournament_size
-            )
+            self.toolbox.register("select", tools.selTournament, tournsize=self.tournament_size)
         else:
             self.toolbox.register("select", tools.selRoulette)
 
@@ -1105,9 +1043,7 @@ class GAFeatureSelectionCV(BaseSearchCV):
             current_generation_params[f"test_{metric}"] = cv_results[f"test_{metric}"]
 
             if self.return_train_score:
-                current_generation_params[f"train_{metric}"] = cv_results[
-                    f"train_{metric}"
-                ]
+                current_generation_params[f"train_{metric}"] = cv_results[f"train_{metric}"]
 
         index = len(self.logbook.chapters["parameters"])
         current_generation_features = {"index": index, **current_generation_params}
@@ -1128,6 +1064,7 @@ class GAFeatureSelectionCV(BaseSearchCV):
         """
         Main method of GAFeatureSelectionCV, starts the optimization
         procedure with to find the best features set
+
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
@@ -1172,14 +1109,13 @@ class GAFeatureSelectionCV(BaseSearchCV):
         self._register()
 
         # Optimization routine from the selected evolutionary algorithm
-        pop, log, n_gen = self._select_algorithm(
-            pop=self._pop, stats=self._stats, hof=self._hof
-        )
+        pop, log, n_gen = self._select_algorithm(pop=self._pop, stats=self._stats, hof=self._hof)
 
         # Update the _n_iterations value as the algorithm could stop earlier due a callback
         self._n_iterations = n_gen
 
         self.best_features_ = np.array(self._hof[0], dtype=bool)
+        self.support_ = self.best_features_
 
         self.cv_results_ = create_feature_selection_cv_results_(
             logbook=self.logbook,
@@ -1204,6 +1140,7 @@ class GAFeatureSelectionCV(BaseSearchCV):
             self.refit_time_ = refit_end_time - refit_start_time
 
             self.best_estimator_ = self.estimator
+            self.estimator_ = self.best_estimator_
 
         self.hof = self._hof
 
@@ -1234,40 +1171,9 @@ class GAFeatureSelectionCV(BaseSearchCV):
             The number of generations that the evolutionary algorithm ran
         """
 
-        if self.algorithm == Algorithms.eaSimple.value:
-
-            pop, log, gen = eaSimple(
-                pop,
-                self.toolbox,
-                cxpb=self.crossover_adapter,
-                stats=stats,
-                mutpb=self.mutation_adapter,
-                ngen=self.generations,
-                halloffame=hof,
-                callbacks=self.callbacks,
-                verbose=self.verbose,
-                estimator=self,
-            )
-
-        elif self.algorithm == Algorithms.eaMuPlusLambda.value:
-
-            pop, log, gen = eaMuPlusLambda(
-                pop,
-                self.toolbox,
-                mu=self.population_size,
-                lambda_=2 * self.population_size,
-                cxpb=self.crossover_adapter,
-                stats=stats,
-                mutpb=self.mutation_adapter,
-                ngen=self.generations,
-                halloffame=hof,
-                callbacks=self.callbacks,
-                verbose=self.verbose,
-                estimator=self,
-            )
-
-        elif self.algorithm == Algorithms.eaMuCommaLambda.value:
-            pop, log, gen = eaMuCommaLambda(
+        selected_algorithm = algorithms_factory.get(self.algorithm, None)
+        if selected_algorithm:
+            pop, log, gen = selected_algorithm(
                 pop,
                 self.toolbox,
                 mu=self.population_size,
@@ -1355,3 +1261,147 @@ class GAFeatureSelectionCV(BaseSearchCV):
         self.generations otherwise
         """
         return self._n_iterations
+
+    def _check_refit_for_multimetric(self, scores):  # pragma: no cover
+        """Check `refit` is compatible with `scores` is valid"""
+        multimetric_refit_msg = (
+            "For multi-metric scoring, the parameter refit must be set to a "
+            "scorer key or a callable to refit an estimator with the best "
+            "parameter setting on the whole data and make the best_* "
+            "attributes available for that metric. If this is not needed, "
+            f"refit should be set to False explicitly. {self.refit!r} was "
+            "passed."
+        )
+
+        valid_refit_dict = isinstance(self.refit, str) and self.refit in scores
+
+        if self.refit is not False and not valid_refit_dict and not callable(self.refit):
+            raise ValueError(multimetric_refit_msg)
+
+    @property
+    def n_features_in_(self):  # pragma: no cover
+        """Number of features seen during `fit`."""
+        # For consistency with other estimators we raise a AttributeError so
+        # that hasattr() fails if the estimator isn't fitted.
+        if not self._fitted:
+            raise AttributeError(
+                "{} object has no n_features_in_ attribute.".format(self.__class__.__name__)
+            )
+
+        return self.n_features
+
+    def _get_support_mask(self):
+        if not self._fitted:
+            raise NotFittedError(
+                f"This GAFeatureSelectionCV instance is not fitted yet "
+                f"or used refit=False. Call 'fit' with appropriate "
+                f"arguments before using this estimator."
+            )
+        return self.best_features_
+
+    @available_if(_estimator_has("decision_function"))
+    def decision_function(self, X):
+        """Call decision_function on the estimator with the best found features.
+       Only available if ``refit=True`` and the underlying estimator supports
+       ``decision_function``.
+
+       Parameters
+       ----------
+       X : indexable, length n_samples
+           Must fulfill the input assumptions of the
+           underlying estimator.
+
+       Returns
+       -------
+       y_score : ndarray of shape (n_samples,) or (n_samples, n_classes) \
+               or (n_samples, n_classes * (n_classes-1) / 2)
+           Result of the decision function for `X` based on the estimator with
+           the best found parameters.
+       """
+        return self.estimator.decision_function(self.transform(X))
+
+    @available_if(_estimator_has("predict"))
+    def predict(self, X):
+        """Call predict on the estimator with the best found features.
+        Only available if ``refit=True`` and the underlying estimator supports
+        ``predict``.
+
+        Parameters
+        ----------
+        X : indexable, length n_samples
+            Must fulfill the input assumptions of the
+            underlying estimator.
+
+        Returns
+        -------
+        y_pred : ndarray of shape (n_samples,)
+            The predicted labels or values for `X` based on the estimator with
+            the best found parameters.
+        """
+        return self.estimator.predict(self.transform(X))
+
+    @available_if(_estimator_has("predict_log_proba"))
+    def predict_log_proba(self, X):
+        """Call predict_log_proba on the estimator with the best found features.
+        Only available if ``refit=True`` and the underlying estimator supports
+        ``predict_log_proba``.
+
+        Parameters
+        ----------
+        X : indexable, length n_samples
+            Must fulfill the input assumptions of the
+            underlying estimator.
+
+        Returns
+        -------
+        y_pred : ndarray of shape (n_samples,) or (n_samples, n_classes)
+            Predicted class log-probabilities for `X` based on the estimator
+            with the best found parameters. The order of the classes
+            corresponds to that in the fitted attribute :term:`classes_`.
+        """
+        return self.estimator.predict_log_proba(self.transform(X))
+
+    @available_if(_estimator_has("predict_proba"))
+    def predict_proba(self, X):
+        """Call predict_proba on the estimator with the best found features.
+        Only available if ``refit=True`` and the underlying estimator supports
+        ``predict_proba``.
+
+        Parameters
+        ----------
+        X : indexable, length n_samples
+            Must fulfill the input assumptions of the
+            underlying estimator.
+
+        Returns
+        -------
+        y_pred : ndarray of shape (n_samples,) or (n_samples, n_classes)
+            Predicted class probabilities for `X` based on the estimator with
+            the best found parameters. The order of the classes corresponds
+            to that in the fitted attribute :term:`classes_`.
+        """
+        return self.estimator.predict_proba(self.transform(X))
+
+    @available_if(_estimator_has("score"))
+    def score(self, X, y):
+        """Return the score on the given data, if the estimator has been refit.
+        This uses the score defined by ``scoring`` where provided, and the
+        ``best_estimator_.score`` method otherwise.
+
+        Parameters
+        ----------
+        X : array-like of shape (n_samples, n_features)
+            Input data, where `n_samples` is the number of samples and
+            `n_features` is the number of features.
+        y : array-like of shape (n_samples, n_output) \
+            or (n_samples,), default=None
+            Target relative to X for classification or regression;
+            None for unsupervised learning.
+
+        Returns
+        -------
+        score : float
+            The score defined by ``scoring`` if provided, and the
+            ``best_estimator_.score`` method otherwise.
+        """
+        return self.estimator.score(self.transform(X), y)
