@@ -1,5 +1,6 @@
 import numpy as np
 from scipy import stats
+import random
 
 from .space_parameters import (
     IntegerDistributions,
@@ -13,7 +14,11 @@ class Integer(BaseDimension):
     """class for hyperparameters search space of integer values"""
 
     def __init__(
-        self, lower: int = None, upper: int = None, distribution: str = "uniform", random_state=None
+        self,
+        lower: int = None,
+        upper: int = None,
+        distribution: str = "uniform",
+        random_state=None,
     ):
         """
         Parameters
@@ -63,7 +68,11 @@ class Continuous(BaseDimension):
     """class for hyperparameters search space of real values"""
 
     def __init__(
-        self, lower: float = None, upper: float = None, distribution: str = "uniform", random_state=None
+        self,
+        lower: float = None,
+        upper: float = None,
+        distribution: str = "uniform",
+        random_state=None,
     ):
         """
         Parameters
@@ -117,7 +126,11 @@ class Categorical(BaseDimension):
     """class for hyperparameters search space of categorical values"""
 
     def __init__(
-        self, choices: list = None, priors: list = None, distribution: str = "choice", random_state=None
+        self,
+        choices: list = None,
+        priors: list = None,
+        distribution: str = "choice",
+        random_state=None,
     ):
         """
         Parameters
@@ -156,15 +169,16 @@ class Categorical(BaseDimension):
         self.choices = choices
         self.distribution = distribution
         self.random_state = random_state
+        random.seed(random_state)
         self.rng = None if not self.random_state else np.random.default_rng(self.random_state)
 
         if self.distribution == CategoricalDistributions.choice.value:
-            self.rvs = self.rng.choice if self.rng else np.random.choice
+            self.rvs = self.rng.choice if self.rng else random.choice
 
     def sample(self):
         """Sample a random value from the assigned distribution"""
 
-        return self.rvs(self.choices, p=self.priors)
+        return self.rvs(self.choices)
 
 
 def check_space(param_grid: dict = None):
@@ -233,5 +247,4 @@ class Space(object):
         return self.dimensions
 
     def __getitem__(self, index):
-
         return self.param_grid[index]
